-- =============================================================================================================
-- *
-- * Copyright (c) M.Freeman
-- *
-- * File Name: mc_cu.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: MicroCoded Control Unit
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes: Simple micro coded controller
-- *
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY mc_cu IS
PORT (
  clk           : IN STD_LOGIC;
  clr           : IN STD_LOGIC;
  acc_en        : OUT STD_LOGIC;    
  acc_rst       : OUT STD_LOGIC;  
  pc_en         : OUT STD_LOGIC;  
  pc_rst        : OUT STD_LOGIC;  
  ir_en         : OUT STD_LOGIC;  
  ir_rst        : OUT STD_LOGIC;  
  ir_dout       : IN STD_LOGIC_VECTOR( 15 DOWNTO 0 );
  psw_en        : OUT STD_LOGIC;  
  psw_rst       : OUT STD_LOGIC;  
  psw_dout      : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  operand_A_sel : OUT STD_LOGIC;
  operand_B_sel : OUT STD_LOGIC;
  addr_sel      : OUT STD_LOGIC;  
  alu_sel       : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 );
  alu_cin       : OUT STD_LOGIC;
  rw            : OUT STD_LOGIC
); 
END mc_cu;

ARCHITECTURE mc_cu_arch OF mc_cu IS

  -- COMPONENTS
  
  COMPONENT reg 
  GENERIC (
    width : INTEGER := 32 );
  PORT ( 
    clk : IN STD_LOGIC;
    clr : IN STD_LOGIC;
    en : IN STD_LOGIC;
    rst : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    dout : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0));
  END COMPONENT;
  
  COMPONENT mc_rom 
  GENERIC(
    SIMULATION : BOOLEAN := TRUE;
    ADDR_WIDTH : INTEGER := 8;
    DATA_WIDTH : INTEGER := 30 );
  PORT (
    addr: IN STD_LOGIC_VECTOR ( ADDR_WIDTH-1  DOWNTO 0 );
    data: OUT STD_LOGIC_VECTOR ( DATA_WIDTH-1 DOWNTO 0 ) );
  END COMPONENT;
  
  COMPONENT addr_rom 
  GENERIC(
    SIMULATION : BOOLEAN := TRUE;
    ADDR_WIDTH : INTEGER := 4;
    DATA_WIDTH : INTEGER := 8 );
  PORT (
    addr: IN STD_LOGIC_VECTOR ( ADDR_WIDTH-1  DOWNTO 0 );
    data: OUT STD_LOGIC_VECTOR ( DATA_WIDTH-1 DOWNTO 0 ) );
  END COMPONENT;

  -- SIGNALS
  
  SIGNAL VCC : STD_LOGIC;
  SIGNAL GND : STD_LOGIC;
  
  SIGNAL mc_ir_dat : STD_LOGIC_VECTOR ( 29 DOWNTO 0 );
  
  SIGNAL mc_rom_addr : STD_LOGIC_VECTOR ( 7 DOWNTO 0 );
  SIGNAL mc_rom_data : STD_LOGIC_VECTOR ( 29 DOWNTO 0 );
    
  SIGNAL addr_rom_addr : STD_LOGIC_VECTOR ( 3 DOWNTO 0 );
  SIGNAL addr_rom_data : STD_LOGIC_VECTOR ( 7 DOWNTO 0 );
  
  SIGNAL psw_dout_int  : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  
BEGIN

  --
  -- BUFFERS
  --
  
  VCC <= '1';
  GND <= '0';
  
  addr_rom_addr <= ir_dout( 15 DOWNTO 12 ); 
  
  acc_en        <=  mc_ir_dat(0);
  acc_rst       <=  mc_ir_dat(1); 
  
  pc_en         <=  mc_ir_dat(2) WHEN mc_ir_dat(27)='0' ELSE
                    '1'          WHEN ir_dout( 11 DOWNTO 10 )="11" and psw_dout_int(2)='1' ELSE
                    '1'          WHEN ir_dout( 11 DOWNTO 10 )="01" and psw_dout_int(3)='1' ELSE
                    '1'          WHEN ir_dout( 11 DOWNTO 10 )="10" and psw_dout_int(0)='1' ELSE
                    '1'          WHEN ir_dout( 11 DOWNTO 10 )="00" and psw_dout_int(1)='1' ELSE
                    '0';
                    
  pc_rst        <=  mc_ir_dat(3); 
  ir_en         <=  mc_ir_dat(4); 
  ir_rst        <=  mc_ir_dat(5); 
  psw_en        <=  mc_ir_dat(6); 
  psw_rst       <=  mc_ir_dat(7); 
  operand_A_sel <=  mc_ir_dat(8);
  operand_B_sel <=  mc_ir_dat(9);
  addr_sel      <=  mc_ir_dat(10);
  alu_sel       <=  mc_ir_dat(13 DOWNTO 11);
  
  alu_cin       <= '0'             WHEN mc_ir_dat(15 DOWNTO 14)="00" ELSE 
                   '1'             WHEN mc_ir_dat(15 DOWNTO 14)="01" ELSE 
                   not psw_dout(0) WHEN mc_ir_dat(15 DOWNTO 14)="10" ELSE 
                   psw_dout(0); 
                    
  rw            <= not mc_ir_dat(16); 
  
  mc_rom_addr   <= addr_rom_data WHEN mc_ir_dat(26 DOWNTO 25)="01" ELSE  
                   "00000000"    WHEN mc_ir_dat(27)='1' and ir_dout( 11 DOWNTO 10 )="11" and psw_dout_int(2)='1' ELSE
                   "00000000"    WHEN mc_ir_dat(27)='1' and ir_dout( 11 DOWNTO 10 )="01" and psw_dout_int(3)='1' ELSE
                   "00000000"    WHEN mc_ir_dat(27)='1' and ir_dout( 11 DOWNTO 10 )="10" and psw_dout_int(0)='1' ELSE
						 "00000000"    WHEN mc_ir_dat(27)='1' and ir_dout( 11 DOWNTO 10 )="00" and psw_dout_int(1)='1' ELSE
                                 mc_ir_dat( 24 DOWNTO 17 );     
						 
  psw_dout_int  <= psw_dout; --"00000000" WHEN mc_ir_dat(26 DOWNTO 25)="00" ELSE psw_dout;

  --
  -- COMPONENTS
  --
  
  mc_ir_reg : reg 
  GENERIC MAP(
    width => 30 )
  PORT MAP( 
    clk  => clk,
    clr  => clr,
    en   => VCC,
    rst  => GND,
    din  => mc_rom_data,
    dout => mc_ir_dat );
  
  mc_rom_inst : mc_rom
  GENERIC MAP(
    SIMULATION => TRUE,
    ADDR_WIDTH => 8,
    DATA_WIDTH => 30 )
  PORT MAP(
    addr => mc_rom_addr, 
    data => mc_rom_data );
    
  addr_rom_inst : addr_rom
  GENERIC MAP(
    SIMULATION => TRUE,
    ADDR_WIDTH => 4,
    DATA_WIDTH => 8 )
  PORT MAP(
    addr => addr_rom_addr,
    data => addr_rom_data );
    
  --
  -- SIMULATION ONLY
  --
  
  simulation : PROCESS( mc_ir_dat, 
                        ir_dout,
                        psw_dout 
                      )
    VARIABLE status      : string( 1 TO 14 );
    VARIABLE cycle       : string( 1 TO 9 );
    VARIABLE instruction : string( 1 TO 15 );
    VARIABLE temp        : string( 1 TO 4 );

    FUNCTION hexcharacter (nibble: STD_LOGIC_VECTOR(3 DOWNTO 0)) RETURN character IS
      VARIABLE hex: character;
    BEGIN
      CASE nibble IS
        WHEN "0000" => hex := '0';
        WHEN "0001" => hex := '1';
        WHEN "0010" => hex := '2';
        WHEN "0011" => hex := '3';
        WHEN "0100" => hex := '4';
        WHEN "0101" => hex := '5';
        WHEN "0110" => hex := '6';
        WHEN "0111" => hex := '7';
        WHEN "1000" => hex := '8';
        WHEN "1001" => hex := '9';
        WHEN "1010" => hex := 'A';
        WHEN "1011" => hex := 'B';
        WHEN "1100" => hex := 'C';
        WHEN "1101" => hex := 'D';
        WHEN "1110" => hex := 'E';
        WHEN "1111" => hex := 'F';
        WHEN OTHERS => hex := 'x';
      END CASE;
      RETURN hex;
    END hexcharacter;
    
  BEGIN
  
    CASE mc_ir_dat( 26 DOWNTO 25 ) IS
      WHEN "00"   => cycle := "Fetch    ";
      WHEN "01"   => cycle := "Decode   ";
      WHEN "10"   => cycle := "Execute  ";
      WHEN "11"   => cycle := "Increment";
      WHEN OTHERS => cycle := "Unknown  ";
    END CASE;
    
    temp( 1 TO 2 ) := "0x";
    temp(3)        := hexcharacter( ir_dout( 7 DOWNTO 4) );
    temp(4)        := hexcharacter( ir_dout( 3 DOWNTO 0) );
    
    CASE ir_dout( 15 DOWNTO 12 ) IS
      WHEN "0000" => instruction := "Load ACC " & temp & "  ";
      WHEN "0100" => instruction := "Add ACC " & temp & "   ";
      WHEN "1100" => instruction := "Add MEM " & temp & "   ";
      WHEN "1010" => instruction := "Input ACC " & temp & " ";
      WHEN "1000" => instruction := "Jump U " & temp &"    ";
      WHEN "1001" =>
        CASE ir_dout( 11 DOWNTO 10 ) IS
          WHEN "00"   => instruction := "Jump Z " & temp &"    ";
          WHEN "01"   => instruction := "Jump NZ " & temp &"   ";    
          WHEN "10"   => instruction := "Jump C " & temp &"    ";
          WHEN "11"   => instruction := "Jump NC " & temp &"   ";
          WHEN OTHERS => instruction := "Unknown        ";
        END CASE;
      WHEN "1110" => instruction := "Output ACC " & temp & "";
      WHEN OTHERS => instruction := "Unknown        ";
    END CASE;
    
    IF clr='1' 
    THEN
      status := "NZ, NC, Reset ";
    ELSE
      status(7 TO 14) := "        ";
      IF psw_dout(1)='1' 
      THEN
        status(1 TO 4) := " Z, ";
      ELSE
        status(1 TO 4) := "NZ, ";
      END IF;
          
      IF psw_dout(0)='1'  
      THEN
        status(5 TO 6) := " C";
      ELSE
        status(5 TO 6) := "NC";
      END IF;
    END IF;
    
  END PROCESS;
  
  
END mc_cu_arch;
