-- =============================================================================================================
-- *
-- * Copyright (c) M.Freeman
-- *
-- * File Name: mc_rom.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Micro code ROM
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *   
-- =============================================================================================================
--
-- INSTRUCTIONS
-- ------------
-- Load   ACC kk : 0000 XXXX KKKKKKKK
-- Add    ACC kk : 0100 XXXX KKKKKKKK
-- Add    ACC pp : 1100 XXXX PPPPPPPP
-- Input  ACC pp : 1010 XXXX PPPPPPPP
-- Output ACC pp : 1110 XXXX PPPPPPPP
-- Jump   U   aa : 1000 XXXX AAAAAAAA
-- Jump   Z   aa : 1001 00XX AAAAAAAA
-- Jump   C   aa : 1001 10XX AAAAAAAA
-- Jump  NZ   aa : 1001 01XX AAAAAAAA
-- Jump  NC   aa : 1001 11XX AAAAAAAA
--
-- STATE CODES             ALU_CIN
-- -----------             -------
-- 00 : FETCH              00 : 0
-- 01 : DECODE             01 : 1
-- 10 : EXECUTE            10 : not cout
-- 11 : INCREMENT          11 : cout 
--
-- MICRO-INSTRUCTIONS
-- ------------------
--                      29-28 27   26-25 24-17       16 15-14   13-11   10       9        8        7       6      5      4     3      2     1       0
--                      code  cond state mc_rom_addr rw alu_cin alu_sel addr_sel op_B_sel op_A_sel psw_rst psw_en ir_rst ir_en pc_rst pc_en acc_rst acc_en
-- 00 : Fetch_S0        00    0    00    00000001    0  00      000     0        0        0        0       0      0      1     0      0     0       0
-- 01 : Decode_S0       00    0    01    00000000    0  00      000     0        0        0        0       0      0      0     0      0     0       0
-- 02 : Inc_S0          00    0    11    00000000    0  01      000     0        0        1        0       0      0      0     0      1     0       0
--
-- 10 : Load   ACC kk   00    0    10    00000010    0  01      011     0        0        0        0       0      0      0     0      0     0       1
-- 11 : Add    ACC kk   00    0    10    00000010    0  00      001     0        1        0        0       1      0      0     0      0     0       1
-- 12 : Input  ACC pp   00    0    10    00010011    0  00      000     1        0        0        0       0      0      0     0      0     0       0 
-- 13                   00    0    10    00000010    0  00      000     1        0        0        0       0      0      0     0      0     0       1 
-- 14 : Jump   XX  aa   00    1    10    00000010    0  01      011     0        0        0        0       0      0      0     0      0     0       0             
-- 15 : Output ACC pp   00    0    10    00010110    0  00      000     1        0        0        0       0      0      0     0      0     0       0
-- 16                   00    0    10    00010111    1  00      000     1        0        0        0       0      0      0     0      0     0       0
-- 17                   00    0    10    00000010    0  00      000     1        0        0        0       0      0      0     0      0     0       0
-- 18 : Jump    U  aa   00    0    10    00000000    0  01      011     0        0        0        0       0      0      0     0      1     0       0     
-- 19 : Add    ACC pp   00    0    10    00011010    0  00      000     1        0        0        0       0      0      0     0      0     0       0 
-- 1A                   00    0    10    00011011    0  00      000     1        0        0        0       0      0      1     0      0     0       0 
-- 1B                   00    0    10    00000010    0  00      001     0        1        0        0       0      0      0     0      0     0       1 
--

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY mc_rom IS
GENERIC(
  SIMULATION : BOOLEAN := TRUE;
  ADDR_WIDTH : INTEGER := 8;
  DATA_WIDTH : INTEGER := 26 );
PORT (
  addr: IN STD_LOGIC_VECTOR ( ADDR_WIDTH-1  DOWNTO 0 );
  data: OUT STD_LOGIC_VECTOR ( DATA_WIDTH-1 DOWNTO 0 )
);
END mc_rom;

ARCHITECTURE mc_rom_arch OF mc_rom IS

  CONSTANT INVALID_DATA : STD_LOGIC_VECTOR ((DATA_WIDTH-1) DOWNTO 0) := (OTHERS => 'X');
  
  subtype DATA_ROM_WORD IS STD_LOGIC_VECTOR ((DATA_WIDTH-1) DOWNTO 0);
  TYPE DATA_ROM_TABLE IS ARRAY (0 TO (2**ADDR_WIDTH)-1) OF DATA_ROM_WORD; 


  CONSTANT DATA_ROM: DATA_ROM_TABLE := DATA_ROM_TABLE'        
  ( DATA_ROM_WORD'("000000000000100000000000010000"), --00  
    DATA_ROM_WORD'("000010000000000000000000000000"), --01	 			
    DATA_ROM_WORD'("000110000000000100000100000100"), --02	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --03	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --04	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --05	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --06	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --07	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --08	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --09	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --0A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --0B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --0C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --0D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --0E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --0F	 

    DATA_ROM_WORD'("000100000001000101100000000001"), --10  
    DATA_ROM_WORD'("000100000001000000101001000001"), --11	 			
    DATA_ROM_WORD'("000100001001100000010000000000"), --12	 			
    DATA_ROM_WORD'("000100000001000000010000000001"), --13	 			
    DATA_ROM_WORD'("001100000001000101100000000000"), --14	 			
    DATA_ROM_WORD'("000100001011000000010000000000"), --15	 			
    DATA_ROM_WORD'("000100001011110000010000000000"), --16	 			
    DATA_ROM_WORD'("000100000001000000010000000000"), --17	 			
    DATA_ROM_WORD'("000100000000000101100000000100"), --18	 			
    DATA_ROM_WORD'("000100001101000000010000000000"), --19	 			
    DATA_ROM_WORD'("000100001101100000010000010000"), --1A	 			
    DATA_ROM_WORD'("000100000001000000101000000001"), --1B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --1C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --1D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --1E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --1F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --20  
    DATA_ROM_WORD'("000000000000000000000000000000"), --21	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --22	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --23	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --24	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --25	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --26	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --27	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --28	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --29	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --2A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --2B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --2C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --2D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --2E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --2F	
   
    DATA_ROM_WORD'("000000000000000000000000000000"), --30  
    DATA_ROM_WORD'("000000000000000000000000000000"), --31	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --32	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --33	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --34	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --35	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --36	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --37	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --38	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --39	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --3A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --3B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --3C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --3D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --3E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --3F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --40  
    DATA_ROM_WORD'("000000000000000000000000000000"), --41	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --42	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --43	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --44	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --45	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --46	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --47	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --48	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --49	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --4A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --4B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --4C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --4D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --4E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --4F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --50  
    DATA_ROM_WORD'("000000000000000000000000000000"), --51	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --52	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --53	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --54	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --55	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --56	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --57	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --58	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --59	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --5A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --5B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --5C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --5D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --5E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --5F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --60  
    DATA_ROM_WORD'("000000000000000000000000000000"), --61	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --62	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --63	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --64	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --65	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --66	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --67	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --68	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --69	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --6A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --6B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --6C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --6D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --6E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --6F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --70  
    DATA_ROM_WORD'("000000000000000000000000000000"), --71	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --72	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --73	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --74	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --75	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --76	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --77	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --78	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --79	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --7A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --7B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --7C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --7D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --7E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --7F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --80  
    DATA_ROM_WORD'("000000000000000000000000000000"), --81	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --82	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --83	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --84	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --85	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --86	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --87	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --88	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --89	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --8A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --8B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --8C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --8D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --8E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --8F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --90  
    DATA_ROM_WORD'("000000000000000000000000000000"), --91	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --92	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --93	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --94	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --95	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --96	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --97	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --98	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --99	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --9A	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --9B	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --9C	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --9D	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --9E	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --9F	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --A0  
    DATA_ROM_WORD'("000000000000000000000000000000"), --A1	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A2	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A3	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A4	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A5	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A6	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A7	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A8	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --A9	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --AA	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --AB	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --AC	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --AD	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --AE	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --AF	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --B0  
    DATA_ROM_WORD'("000000000000000000000000000000"), --B1	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B2	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B3	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B4	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B5	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B6	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B7	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B8	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --B9	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --BA	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --BB	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --BC	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --BD	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --BE	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --BF	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --C0  
    DATA_ROM_WORD'("000000000000000000000000000000"), --C1	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C2	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C3	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C4	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C5	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C6	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C7	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C8	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --C9	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --CA	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --CB	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --CC	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --CD	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --CE	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --CF	
   
    DATA_ROM_WORD'("000000000000000000000000000000"), --D0  
    DATA_ROM_WORD'("000000000000000000000000000000"), --D1	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D2	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D3	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D4	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D5	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D6	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D7	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D8	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --D9	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --DA	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --DB	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --DC	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --DD	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --DE	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --DF	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --E0  
    DATA_ROM_WORD'("000000000000000000000000000000"), --E1	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --E2	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --E3	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --E4	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --E5	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --E6	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --E7	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --8	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --E9	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --EA	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --EB	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --EC	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --ED	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --EE	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --EF	
    
    DATA_ROM_WORD'("000000000000000000000000000000"), --F0  
    DATA_ROM_WORD'("000000000000000000000000000000"), --F1	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F2	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F3	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F4	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F5	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F6	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F7	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F8	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --F9	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --FA	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --FB	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --FC	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --FD	 			
    DATA_ROM_WORD'("000000000000000000000000000000"), --FE	 			
    DATA_ROM_WORD'("000000000000000000000000000000")  --FF	

  );
    
BEGIN
  PROCESS ( addr )
  BEGIN
    IF SIMULATION             
    THEN
      IF is_x( addr )                             -- Simulation version
      THEN
        data <= INVALID_DATA after 100 ps;
      ELSE
        data <= data_rom( CONV_INTEGER(addr)) after 100 ps;
      END IF;
    ELSE  
      data <= data_rom( CONV_INTEGER(addr));      -- Implementstion version
    END IF; 	
  END PROCESS;

END mc_rom_arch;
