-- =============================================================================================================
-- *
-- * Copyright (c) M.Freeman
-- *
-- * File Name: mcp.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: Mikes (minimal) CISC Processor
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes: A very simple CISC processor implementation of the Von-Neumann processor discussed in lecture
-- *        slides. Simulation process base on PicoBlaze simulation process
-- *   
-- =============================================================================================================

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY mcp IS
PORT ( 
  clk  : IN STD_LOGIC;
  clr  : IN STD_LOGIC;
  addr : OUT STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  din  : IN  STD_LOGIC_VECTOR( 15 DOWNTO 0);
  dout : OUT STD_LOGIC_VECTOR( 15 DOWNTO 0);
  rw   : OUT STD_LOGIC
);
END mcp;

ARCHITECTURE mcp_arch OF mcp IS

  --
  -- COMPONENTS
  --
  
  COMPONENT mux_2
  GENERIC (
    width : INTEGER := 32 );
  PORT ( 
    din_a : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    din_b : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);  
    dout  : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    sel   : IN STD_LOGIC );
  END COMPONENT;
  
  COMPONENT reg
  GENERIC (
    width : INTEGER := 32 );
  PORT ( 
    clk  : IN STD_LOGIC;
    clr  : IN STD_LOGIC;
    en   : IN STD_LOGIC;
    rst  : IN STD_LOGIC;
    din  : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    dout : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0));
  END COMPONENT;
  
  COMPONENT alu8bit 
  PORT (
    a    : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    b    : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    cin  : IN STD_LOGIC;
    s2   : IN STD_LOGIC;
    s1   : IN STD_LOGIC;
    s0   : IN STD_LOGIC;
    f    : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cout : OUT STD_LOGIC );
  END COMPONENT;
  
  COMPONENT wide_nor_gate 
  GENERIC (
    WIDTH : INTEGER := 7 );
  PORT (
    dat_i : IN  STD_LOGIC_VECTOR( WIDTH-1 DOWNTO 0 );
    dat_o : OUT STD_LOGIC );
  END COMPONENT;
  
  COMPONENT mc_cu
  PORT (
    clk           : IN STD_LOGIC;
    clr           : IN STD_LOGIC;
    acc_en        : OUT STD_LOGIC;    
    acc_rst       : OUT STD_LOGIC;  
    pc_en         : OUT STD_LOGIC;  
    pc_rst        : OUT STD_LOGIC;  
    ir_en         : OUT STD_LOGIC;  
    ir_rst        : OUT STD_LOGIC;  
    ir_dout       : IN STD_LOGIC_VECTOR( 15 DOWNTO 0 );
    psw_en        : OUT STD_LOGIC;  
    psw_rst       : OUT STD_LOGIC;  
    psw_dout      : IN STD_LOGIC_VECTOR( 7 DOWNTO 0 );
    operand_A_sel : OUT STD_LOGIC;
    operand_B_sel : OUT STD_LOGIC;
    addr_sel      : OUT STD_LOGIC;  
    alu_sel       : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 );
    alu_cin       : OUT STD_LOGIC;  
    rw            : OUT STD_LOGIC ); 
  END COMPONENT;

  --
  -- SIGNALS
  --
  
  SIGNAL acc_en   : STD_LOGIC;
  SIGNAL acc_rst  : STD_LOGIC;
  SIGNAL acc_din  : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  SIGNAL acc_dout : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  
  SIGNAL pc_en   : STD_LOGIC;
  SIGNAL pc_rst  : STD_LOGIC;
  SIGNAL pc_din  : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  SIGNAL pc_dout : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  
  SIGNAL ir_en   : STD_LOGIC;
  SIGNAL ir_rst  : STD_LOGIC;
  SIGNAL ir_din  : STD_LOGIC_VECTOR( 15 DOWNTO 0 );
  SIGNAL ir_dout : STD_LOGIC_VECTOR( 15 DOWNTO 0 );

  SIGNAL psw_en   : STD_LOGIC;
  SIGNAL psw_rst  : STD_LOGIC;
  SIGNAL psw_din  : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  SIGNAL psw_dout : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  
  SIGNAL operand_A     : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  SIGNAL operand_A_sel : STD_LOGIC;
  
  SIGNAL operand_B     : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  SIGNAL operand_B_sel : STD_LOGIC; 
  
  SIGNAL addr_sel : STD_LOGIC; 
  
  SIGNAL alu_sel  : STD_LOGIC_VECTOR( 2 DOWNTO 0 );
  SIGNAL alu_dout : STD_LOGIC_VECTOR( 7 DOWNTO 0 );
  SIGNAL alu_cin  : STD_LOGIC;
  SIGNAL alu_cout : STD_LOGIC;
  
  SIGNAL alu_zero : STD_LOGIC;
  
BEGIN

  --
  -- BUFFERS
  --

  acc_din <= alu_dout;
  dout <= "10000000" & acc_dout;
  
  psw_din <= "0000" & (not alu_zero) & (not alu_cout) & alu_zero & alu_cout;
  ir_din  <= din;
  pc_din  <= alu_dout;
  
  --
  -- REGISTERS
  --
  
  acc : reg
  GENERIC MAP(
    width => 8 )
  PORT MAP( 
    clk  => clk,
    clr  => clr,
    en   => acc_en,
    rst  => acc_rst,
    din  => acc_din,
    dout => acc_dout );
    
  pc : reg
  GENERIC MAP(
    width => 8 )
  PORT MAP( 
    clk  => clk,
    clr  => clr,
    en   => pc_en,
    rst  => pc_rst,
    din  => pc_din,
    dout => pc_dout );
    
  ir : reg
  GENERIC MAP(
    width => 16 )
  PORT MAP( 
    clk  => clk,
    clr  => clr,
    en   => ir_en,
    rst  => ir_rst,
    din  => ir_din,
    dout => ir_dout );
    
  psw : reg
  GENERIC MAP(
    width => 8 )
  PORT MAP( 
    clk  => clk,
    clr  => clr,
    en   => psw_en,
    rst  => psw_rst,
    din  => psw_din,
    dout => psw_dout );
    
  --
  -- MUX
  --
    
  operand_A_mux : mux_2
  GENERIC MAP(
    width => 8 )
  PORT MAP( 
    din_a => ir_dout( 7 DOWNTO 0 ),
    din_b => pc_dout,
    dout  => operand_A,
    sel   => operand_A_sel );
    
  operand_B_mux : mux_2
  GENERIC MAP(
    width => 8 )
  PORT MAP( 
    din_a => din( 7 DOWNTO 0 ),
    din_b => acc_dout,
    dout  => operand_B,
    sel   => operand_B_sel );
    
  addr_mux : mux_2
  GENERIC MAP(
    width => 8 )
  PORT MAP( 
    din_a => pc_dout,
    din_b => operand_A,
    dout  => addr,
    sel   => addr_sel );
    
  --
  -- ALU
  --
    
  alu : alu8bit 
  PORT MAP(
    a    => operand_A,
    b    => operand_B,
    cin  => alu_cin,
    s2   => alu_sel(2),
    s1   => alu_sel(1),
    s0   => alu_sel(0),
    f    => alu_dout,
    cout => alu_cout );
    
  zero : wide_nor_gate 
  GENERIC MAP(
    WIDTH => 8 )
  PORT MAP(
    dat_i  => alu_dout,
    dat_o  => alu_zero );

  --
  -- CONTROL UNIT
  --
  
  mc_controller : mc_cu
  PORT MAP(
    clk           => clk,
    clr           => clr,
    acc_en        => acc_en,  
    acc_rst       => acc_rst,
    pc_en         => pc_en,   
    pc_rst        => pc_rst, 
    ir_en         => ir_en,
    ir_rst        => ir_rst, 
    ir_dout       => ir_dout,
    psw_en        => psw_en,
    psw_rst       => psw_rst,
    psw_dout      => psw_dout,
    operand_A_sel => operand_A_sel,
    operand_B_sel => operand_B_sel,
    addr_sel      => addr_sel,
    alu_sel       => alu_sel,
    alu_cin       => alu_cin,
    rw            => rw );


END mcp_arch;
